@@ -1,12 +1,12 @@
 # flake8: noqa: F401
 
 from .add_config import AddConfig, ChunkerConfig
-from .apps.app_config import AppConfig
+from .app_config import AppConfig
 from .base_config import BaseConfig
+from .cache_config import CacheConfig
 from .embedder.base import BaseEmbedderConfig
 from .embedder.base import BaseEmbedderConfig as EmbedderConfig
 from .llm.base import BaseLlmConfig
-from .pipeline_config import PipelineConfig
 from .vectordb.chroma import ChromaDbConfig
 from .vectordb.elasticsearch import ElasticsearchDBConfig
 from .vectordb.opensearch import OpenSearchDBConfig
