@@ -2,10 +2,10 @@ import importlib
 import os
 from typing import Optional
 
-from langchain.llms import Cohere
+from langchain.llms.cohere import Cohere
 
 from embedchain.config import BaseLlmConfig
-from embedchain.helper.json_serializable import register_deserializable
+from embedchain.helpers.json_serializable import register_deserializable
 from embedchain.llm.base import BaseLlm
 
 
